@@ -21,22 +21,21 @@ limitations under the License.
 from __future__ import annotations
 
 import json
-from typing import Optional, List
+from typing import Callable, Optional, List
 from functools import reduce
 from dataclasses import asdict
-import xml.dom.minidom  # type: ignore
 import requests
 
-from wechaty_grpc.wechaty import (  # type: ignore
+from wechaty_grpc.wechaty import (
     PuppetStub,
 )
 # pylint: disable=E0401
 from grpclib.client import Channel
 # pylint: disable=E0401
-from pyee import AsyncIOEventEmitter  # type: ignore
-from wechaty_puppet.schemas.types import PayloadType    # type: ignore
+from pyee import AsyncIOEventEmitter
+from wechaty_puppet.schemas.types import PayloadType
 
-from wechaty_puppet import (  # type: ignore
+from wechaty_puppet import (
     EventScanPayload,
     ScanStatus,
 
@@ -69,7 +68,7 @@ from wechaty_puppet import (  # type: ignore
     get_logger
 )
 
-from wechaty_puppet.exceptions import (  # type: ignore
+from wechaty_puppet.exceptions import (
     WechatyPuppetConfigurationError,
     WechatyPuppetError,
     WechatyPuppetGrpcError,
@@ -83,10 +82,11 @@ from wechaty_puppet_service.config import (
 )
 from wechaty_puppet_service.utils import (
     extract_host_and_port,
-    test_endpoint
+    ping_endpoint,
+    message_emoticon
 )
 
-log = get_logger('HostiePuppet')
+log = get_logger('PuppetService')
 
 
 def _map_message_type(message_payload: MessagePayload) -> MessagePayload:
@@ -164,27 +164,44 @@ class PuppetService(Puppet):
     """
 
     def __init__(self, options: PuppetOptions, name: str = 'puppet_service'):
-        """init PuppetService from options or envrionment
+        """init PuppetService from options or environment
 
         Args:
             options (PuppetOptions): the configuration of PuppetService
             name (str, optional): [description]. Defaults to 'service_puppet'.
 
         Raises:
-            WechatyPuppetConfigurationError: raise Error when configuraiton occur error
+            WechatyPuppetConfigurationError: raise Error when configuration occur error
         """
+        super().__init__(options, name)
         token, endpoint = get_token(), get_endpoint()
 
-        if not options.token and not token:
-            raise WechatyPuppetConfigurationError(
-                'wechaty-puppet-service: token not found. please set '
-                'environment<WECHATY_PUPPET_SERVICE_TOKEN|TOKEN|token> as token'
+        # 1. checking the endpoint & token configuration
+        if options.end_point and endpoint:
+            log.warning(
+                f'there are two endpoints from options<{options.end_point}> '
+                f'and environment<{endpoint}>, and the first one is adopted'
             )
 
-        options.token = options.token or token
-        options.end_point = options.end_point or endpoint
+        if options.token and token:
+            log.warning(
+                f'there are two tokens from options[adopted]<{options.token}> '
+                f'and environment<{token}>, and the first one is adopted'
+            )
 
-        super().__init__(options, name)
+        options.end_point = options.end_point or endpoint
+        options.token = options.token or token
+
+        if not options.end_point and not options.token:
+            raise WechatyPuppetConfigurationError(
+                'one of <endpoint> and <token> configuration is required, '
+                'you should set environment<WECHATY_PUPPET_SERVICE_TOKEN|TOKEN|token> as token,'
+                'or environment<WECHATY_PUPPET_SERVICE_ENDPOINT|ENDPOINT|endpoint> as endpoint'
+            )
+
+        if options.end_point and options.token:
+            log.warning(f'there are endpoint<{options.end_point}> and token<{options.token}>, '
+                        f'and the endpoint will be used for service ...')
 
         self.channel: Optional[Channel] = None
         self._puppet_stub: Optional[PuppetStub] = None
@@ -233,7 +250,7 @@ class PuppetService(Puppet):
         file_box = FileBox.from_stream(file_stream, name=name)
         return file_box
 
-    def on(self, event_name: str, caller):
+    def on(self, event_name: str, caller: Callable[..., None]) -> None:
         """
         listen event from the wechaty
         :param event_name:
@@ -258,9 +275,6 @@ class PuppetService(Puppet):
         :return:
         """
         response = await self.puppet_stub.contact_list()
-        if response is None:
-            # TODO -> need to refactor the raised error
-            raise WechatyPuppetGrpcError('response is invalid')
         return response.ids
 
     async def tag_contact_delete(self, tag_id: str) -> None:
@@ -281,7 +295,7 @@ class PuppetService(Puppet):
         # wechaty_grpc has not implement this function
         return None
 
-    async def tag_contact_add(self, tag_id: str, contact_id: str):
+    async def tag_contact_add(self, tag_id: str, contact_id: str) -> None:
         """
         add a tag to contact
         :param tag_id:
@@ -291,7 +305,7 @@ class PuppetService(Puppet):
         await self.puppet_stub.tag_contact_add(
             id=tag_id, contact_id=contact_id)
 
-    async def tag_favorite_add(self, tag_id: str, contact_id: str):
+    async def tag_favorite_add(self, tag_id: str, contact_id: str) -> None:
         """
         add a tag to favorite
         :param tag_id:
@@ -300,7 +314,7 @@ class PuppetService(Puppet):
         """
         # wechaty_grpc has not implement this function
 
-    async def tag_contact_remove(self, tag_id: str, contact_id: str):
+    async def tag_contact_remove(self, tag_id: str, contact_id: str) -> None:
         """
         remove a tag from contact
         :param tag_id:
@@ -422,7 +436,7 @@ class PuppetService(Puppet):
 
         return _map_message_type(response)
 
-    async def message_forward(self, to_id: str, message_id: str):
+    async def message_forward(self, to_id: str, message_id: str) -> None:
         """
         forward the message
         :param to_id:
@@ -441,7 +455,7 @@ class PuppetService(Puppet):
             mini_program = await self.message_mini_program(message_id=message_id)
             await self.message_send_mini_program(conversation_id=to_id, mini_program=mini_program)
         elif payload.type == MessageType.MESSAGE_TYPE_EMOTICON:
-            file_box = await self.message_emoticon(message=payload.text)
+            file_box = await message_emoticon(message=payload.text)
             await self.message_send_file(conversation_id=to_id, file=file_box)
         elif payload.type == MessageType.MESSAGE_TYPE_AUDIO:
             raise WechatyPuppetOperationError('Can not support audio message forward')
@@ -471,20 +485,6 @@ class PuppetService(Puppet):
         file_box = FileBox.from_stream(file_stream, name=name)
         return file_box
 
-    async def message_emoticon(self, message: str) -> FileBox:
-        """
-        emoticon from message
-        :param message:
-        :return:
-        """
-        DOMTree = xml.dom.minidom.parseString(message)
-        collection = DOMTree.documentElement
-        file_box = FileBox.from_url(
-            url=collection.getElementsByTagName('emoji')[0].getAttribute('cdnurl'),
-            name=collection.getElementsByTagName('emoji')[0].getAttribute('md5') + '.gif'
-        )
-        return file_box
-
     async def message_contact(self, message_id: str) -> str:
         """
         extract
@@ -540,10 +540,10 @@ class PuppetService(Puppet):
         response = await self.puppet_stub.contact_alias(
             id=contact_id, alias=alias)
         if response.alias is None and alias is None:
-            raise WechatyPuppetGrpcError('can"t get contact<%s> alias' % contact_id)
+            raise WechatyPuppetGrpcError(f'can"t get contact<{contact_id}> alias')
         return response.alias
 
-    async def contact_payload_dirty(self, contact_id: str):
+    async def contact_payload_dirty(self, contact_id: str) -> None:
         """
         mark the contact payload dirty status, and remove it from the cache
         """
@@ -612,7 +612,7 @@ class PuppetService(Puppet):
                 return phone_response.contact_id
         return None
 
-    async def friendship_add(self, contact_id: str, hello: str):
+    async def friendship_add(self, contact_id: str, hello: str) -> None:
         """
         try to add friendship
         :param contact_id:
@@ -638,7 +638,7 @@ class PuppetService(Puppet):
         )
         return response
 
-    async def friendship_accept(self, friendship_id: str):
+    async def friendship_accept(self, friendship_id: str) -> None:
         """
         accept friendship
         :param friendship_id:
@@ -685,7 +685,7 @@ class PuppetService(Puppet):
         )
         return RoomInvitationPayload(**response.to_dict())
 
-    async def room_invitation_accept(self, room_invitation_id: str):
+    async def room_invitation_accept(self, room_invitation_id: str) -> None:
         """
         accept the room invitation
         :param room_invitation_id:
@@ -701,7 +701,7 @@ class PuppetService(Puppet):
         response = await self.puppet_stub.contact_self_qr_code()
         return response.qrcode
 
-    async def contact_self_name(self, name: str):
+    async def contact_self_name(self, name: str) -> None:
         """
         set the name of the contact
         :param name:
@@ -709,7 +709,7 @@ class PuppetService(Puppet):
         """
         await self.puppet_stub.contact_self_name(name=name)
 
-    async def contact_signature(self, signature: str):
+    async def contact_signature(self, signature: str) -> None:
         """
 
         :param signature:
@@ -723,7 +723,7 @@ class PuppetService(Puppet):
         :return:
         """
 
-    async def room_payload_dirty(self, room_id: str):
+    async def room_payload_dirty(self, room_id: str) -> None:
         """
         mark the room payload dirty status, and remove it from the cache
         :param room_id:
@@ -734,7 +734,7 @@ class PuppetService(Puppet):
             room_id
         )
 
-    async def room_member_payload_dirty(self, room_id: str):
+    async def room_member_payload_dirty(self, room_id: str) -> None:
         """
         mark the room-member payload dirty status, and remove it from the cache
         :param room_id:
@@ -763,7 +763,7 @@ class PuppetService(Puppet):
         response = await self.puppet_stub.room_member_list(id=room_id)
         return response.member_ids
 
-    async def room_add(self, room_id: str, contact_id: str):
+    async def room_add(self, room_id: str, contact_id: str) -> None:
         """
         add contact to room
         :param room_id:
@@ -772,7 +772,7 @@ class PuppetService(Puppet):
         """
         await self.puppet_stub.room_add(id=room_id, contact_id=contact_id)
 
-    async def room_delete(self, room_id: str, contact_id: str):
+    async def room_delete(self, room_id: str, contact_id: str) -> None:
         """
         delete contact from room
         :param room_id:
@@ -781,7 +781,7 @@ class PuppetService(Puppet):
         """
         await self.puppet_stub.room_del(id=room_id, contact_id=contact_id)
 
-    async def room_quit(self, room_id: str):
+    async def room_quit(self, room_id: str) -> None:
         """
         quit from room
         :param room_id:
@@ -789,7 +789,7 @@ class PuppetService(Puppet):
         """
         await self.puppet_stub.room_quit(id=room_id)
 
-    async def room_topic(self, room_id: str, new_topic: str):
+    async def room_topic(self, room_id: str, new_topic: str) -> None:
         """
         set/set topic of the room
         :param room_id:
@@ -858,7 +858,7 @@ class PuppetService(Puppet):
         )
         return file_box
 
-    async def dirty_payload(self, payload_type: PayloadType, payload_id: str):
+    async def dirty_payload(self, payload_type: PayloadType, payload_id: str) -> None:
         """
         mark the payload dirty status, and remove it from the cache
         """
@@ -867,27 +867,24 @@ class PuppetService(Puppet):
             id=payload_id
         )
 
-    def _init_puppet(self):
+    def _init_puppet(self) -> None:
         """
         start puppet channel contact_self_qr_code
         """
-        log.info('init puppet')
-        if not self.options.token and not self.options.end_point:
-            raise WechatyPuppetConfigurationError(
-                'Please set a valid envrioment<WECHATY_PUPPET_SERVICE_TOKEN|ENDPOINT|endpoint> or '
-                '<WECHATY_PUPPET_SERVICE_ENDPOINT|ENDPOINT|endpoint>'
-            )
-        # otherwise load them from server by the token
+        log.info('init puppet ...')
+
+        # 1. if there is no endpoint, it should fetch it from chatie server with token
         if not self.options.end_point:
-            # Query the end_point by the token.
-            log.info('There is no endpoint in cache, trying to fetch endpoint with token.')
-            # TODO: the endpoint should be
-            response = requests.get(
-                f'https://api.chatie.io/v0/hosties/{self.options.token}'
-            )
+
+            url = f'https://api.chatie.io/v0/hosties/{self.options.token}'
+            log.info('fetching endpoint from chatie-server: %s', url)
+            response = requests.get(url)
 
             if response.status_code != 200:
-                raise WechatyPuppetGrpcError('service server is invalid ... ')
+                raise WechatyPuppetGrpcError(
+                    'can"t fetch endpoint from chatie server. '
+                    'You can try it later, or make sure that your pc can connect to heroku server '
+                )
 
             data = response.json()
 
@@ -897,10 +894,11 @@ class PuppetService(Puppet):
                 )
             if 'port' not in data:
                 raise WechatyPuppetGrpcError("can't find service server port")
-            log.debug('get puppet ip address : <%s>', data)
-            self.options.end_point = '{ip}:{port}'.format(**data)
 
-        if test_endpoint(self.options.end_point, log) is False:
+            self.options.end_point = f'{data["ip"]}:{data["port"]}'
+            log.debug('endpoint from chatie-server : <%s>', self.options.end_point)
+
+        if ping_endpoint(self.options.end_point) is False:
             raise WechatyPuppetConfigurationError(
                 f"can't not ping endpoint: {self.options.end_point}"
             )
@@ -931,7 +929,7 @@ class PuppetService(Puppet):
         await self._listen_for_event()
         return None
 
-    async def stop(self):
+    async def stop(self) -> None:
         """
         stop the grpc channel connection
         """
@@ -944,7 +942,7 @@ class PuppetService(Puppet):
             self.channel.close()
             self.channel = None
 
-    async def logout(self):
+    async def logout(self) -> None:
         """
         logout the account
         :return:
@@ -959,11 +957,11 @@ class PuppetService(Puppet):
         except Exception as exception:
             log.error('logout() rejection %s', exception)
         finally:
-            payload = EventLogoutPayload(contact_id=self.login_user_id)
+            payload = EventLogoutPayload(contact_id=self.login_user_id, data='logout')
             self._event_stream.emit('logout', payload)
             self.login_user_id = None
 
-    async def login(self, user_id: str):
+    async def login(self, user_id: str) -> None:
         """
         login the account
         :return:
@@ -972,7 +970,7 @@ class PuppetService(Puppet):
         payload = EventLoginPayload(contact_id=user_id)
         self._event_stream.emit('login', payload)
 
-    async def ding(self, data: Optional[str] = ''):
+    async def ding(self, data: Optional[str] = '') -> None:
         """
         set the ding event
         :param data:
@@ -983,7 +981,7 @@ class PuppetService(Puppet):
         await self.puppet_stub.ding(data=data)
 
     # pylint: disable=R0912,R0915
-    async def _listen_for_event(self):
+    async def _listen_for_event(self) -> None:
         """
         listen event from service server with heartbeat
         """
